<?php
// Own namespace
namespace CoreFramework\Controller;

// Import framework stuff
use CoreFramework\Factory\ObjectFactory;
use CoreFramework\Filter\Filterable;
use CoreFramework\Object\BaseFrameworkSystem;
use CoreFramework\Registry\Registry;
use CoreFramework\Registry\Registerable;
use CoreFramework\Request\Requestable;
use CoreFramework\Response\Responseable;

/**
 * A generic controller class. You should extend this base class if you want to
 * write your own controller. You get the advantage that you can use the pre and
 * post filters.
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2017 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class BaseController extends BaseFrameworkSystem implements Registerable {
	// Exception constants
	const EXCEPTION_FILTER_CHAIN_INVALID = 0xf10;

	// Names of controller's own filter chains
	const FILTER_CHAIN_PRE_COMMAND  = 'controller_pre_command';
	const FILTER_CHAIN_POST_COMMAND = 'controller_post_command';

	/**
	 * Generic filter chains
	 */
	private $filterChains = array();

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);

		// Initialize both filter chains
		$this->initFilterChain(self::FILTER_CHAIN_PRE_COMMAND);
		$this->initFilterChain(self::FILTER_CHAIN_POST_COMMAND);

		// Add this controller to the registry
		Registry::getRegistry()->addInstance('controller', $this);
	}

	/**
	 * Executes a command with pre and post filters
	 *
	 * @param	$requestInstance	A Requestable class
	 * @param	$responseInstance	A Responseable class
	 * @return	void
	 */
	public function executeGenericPrePostCommand (Requestable $requestInstance, Responseable $responseInstance) {
		// Get the command instance from the resolver by sending a request instance to the resolver
		$commandInstance = $this->getResolverInstance()->resolveCommandByRequest($requestInstance);

		// Add more filters by the command
		$commandInstance->addExtraFilters($this, $requestInstance);

		// Run the pre filters
		$this->executePreFilters($requestInstance, $responseInstance);

		// This request was valid! :-D
		$requestInstance->requestIsValid();

		// Execute the command
		$commandInstance->execute($requestInstance, $responseInstance);

		// Run the post filters
		$this->executePostFilters($requestInstance, $responseInstance);

		// Flush the response out
		$responseInstance->flushBuffer();
	}

	/**
	 * Handles the given request and response, redirects to login_failed if
	 * UserAuthorizationException is thrown.
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @param	$responseInstance	An instance of a Responseable class
	 * @return	void
	 */
	public function genericHanleRequestLoginFailedRedirect (Requestable $requestInstance, Responseable $responseInstance) {
		// Get the "form action"
		$formAction = $requestInstance->getRequestElement('form');

		// Get command instance from resolver
		$commandInstance = $this->getResolverInstance()->resolveCommand($formAction);

		// Add more filters by the command
		$commandInstance->addExtraFilters($this, $requestInstance);

		// Try to run the pre filters, if auth exceptions come through redirect here
		try {
			// Run the pre filters
			$this->executePreFilters($requestInstance, $responseInstance);
		} catch (UserAuthorizationException $e) {
			// Redirect to main page
			$responseInstance->redirectToConfiguredUrl('login_failed');

			// Exit here
			exit();
		}

		/*
		 * Is the request still valid? Post filters shall only be executed of
		 * the request is valid
		 */
		if ($requestInstance->isRequestValid()) {
			// Execute the command
			$commandInstance->execute($requestInstance, $responseInstance);

			// Execute *very* generic ppost filters
			$this->executePostFilters($requestInstance, $responseInstance);
		} // END - if

		// Flush the buffer out
		$responseInstance->flushBuffer();
	}

	/**
	 * Generic execute of the command: pre and post filters with redirect
	 * but request becomes valid after pre-filters run.
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @param	$responseInstance	An instance of a Responseable class
	 * @return	void
	 */
	public function genericHanleRequestLoginAreaFailedRedirect (Requestable $requestInstance, Responseable $responseInstance) {
		// Get the command instance from the resolver by sending a request instance to the resolver
		$commandInstance = $this->getResolverInstance()->resolveCommandByRequest($requestInstance);

		// Add more filters by the command
		$commandInstance->addExtraFilters($this, $requestInstance);

		// Try to run the pre filters, if auth exceptions come through redirect here
		try {
			// Run the pre filters
			$this->executePreFilters($requestInstance, $responseInstance);
		} catch (UserAuthorizationException $e) {
			// Redirect to main page
			$responseInstance->redirectToConfiguredUrl('login_failed');

			// Exit here
			exit();
		}

		// This request was valid! :-D
		$requestInstance->requestIsValid();

		// Execute the command
		$commandInstance->execute($requestInstance, $responseInstance);

		// Run the pre filters
		$this->executePostFilters($requestInstance, $responseInstance);

		// Flush the response out
		$responseInstance->flushBuffer();
	}

	/**
	 * Private method to initialize a given filter chain
	 *
	 * @param	$filterChain	Name of the filter chain
	 * @return	void
	 */
	protected function initFilterChain ($filterChain) {
		//* DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('CONTROLLER: ' . $filterChain . ' init: START');
		$this->filterChains[$filterChain] = ObjectFactory::createObjectByConfiguredName('filter_chain_class');
		//* DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('CONTROLLER: ' . $filterChain . ' init: FINISHED');
	}

	/**
	 * Adds a filter to a given filter chain
	 *
	 * @param	$filterChain	Chain of the filter
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 * @throws	InvalidFilterChainException	If the filter chain is invalid
	 */
	protected function addFilter ($filterChain, Filterable $filterInstance) {
		//* DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('CONTROLLER: ' . $filterChain . ',' . $filterInstance->__toString(). ' add: START');

		// Test if the filter is there
		if (!isset($this->filterChains[$filterChain])) {
			// Throw an exception here
			throw new InvalidFilterChainException(array($this, $filterChain), self::EXCEPTION_FILTER_CHAIN_INVALID);
		} // END - if

		// Add the filter
		$this->filterChains[$filterChain]->addFilter($filterInstance);
		//* DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('CONTROLLER: ' . $filterChain . ',' . $filterInstance->__toString(). ' add: FINISH');
	}

	/**
	 * Adds a filter to the pre filter chain
	 *
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 */
	public function addPreFilter (Filterable $filterInstance) {
		// Add the pre filter
		$this->addFilter(self::FILTER_CHAIN_PRE_COMMAND, $filterInstance);
	}

	/**
	 * Adds a filter to the post filter chain
	 *
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 */
	public function addPostFilter (Filterable $filterInstance) {
		// Add the post filter
		$this->addFilter(self::FILTER_CHAIN_POST_COMMAND, $filterInstance);
	}

	/**
	 * Add a shutdown filter
	 *
	 * @param	$filterInstance		A Filterable class
	 * @return	void
	 */
	public function addShutdownFilter (Filterable $filterInstance) {
		$this->addFilter('shutdown', $filterInstance);
	}

	/**
	 * Executes given filter chain chain
	 *
	 * @param	$filterChain		Chain of the filter to execute
	 * @param	$requestInstance	An instance of a Requestable class
	 * @param	$responseInstance	An instance of a Responseable class
	 * @return	void
	 * @throws	InvalidFilterChainException	If the filter chain is invalid
	 */
	protected function executeFilters ($filterChain, Requestable $requestInstance, Responseable $responseInstance) {
		// Test if the filter is there
		if (!isset($this->filterChains[$filterChain])) {
			// Throw an exception here
			throw new InvalidFilterChainException(array($this, $filterChain), self::EXCEPTION_FILTER_CHAIN_INVALID);
		} // END - if

		// Run all filters
		$this->filterChains[$filterChain]->processFilters($requestInstance, $responseInstance);
	}

	/**
	 * Executes all pre filters
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @param	$responseInstance	An instance of a Responseable class
	 * @return	void
	 */
	protected function executePreFilters (Requestable $requestInstance, Responseable $responseInstance) {
		// Execute all pre filters
		$this->executeFilters(self::FILTER_CHAIN_PRE_COMMAND, $requestInstance, $responseInstance);
	}

	/**
	 * Executes all post filters
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @param	$responseInstance	An instance of a Responseable class
	 * @return	void
	 */
	protected function executePostFilters (Requestable $requestInstance, Responseable $responseInstance) {
		// Execute all post filters
		$this->executeFilters(self::FILTER_CHAIN_POST_COMMAND, $requestInstance, $responseInstance);
	}

	/**
	 * Executes all shutdown filters
	 *
	 * @param	$requestInstance	A Requestable class
	 * @param	$responseInstance	A Responseable class
	 * @return	void
	 */
	public function executeShutdownFilters (Requestable $requestInstance, Responseable $responseInstance) {
		$this->executeFilters('shutdown', $requestInstance, $responseInstance);
	}

}
